/***********************************************************************
 *      This example program is an excerpt from the book:              *
 *                                                                     *
 *          X Window OSF/Motif Toolkit Programming (I)                 *
 *            - Programming Basics, Resource and Event -               *
 *                                                                     *
 *      Copyright 1994, Hideyuki Hayashi                               *
 *                                                                     *
 *      This program can be distributed without fee, provided          *
 *      that the above copyright notice appear in all copies.          *
 ***********************************************************************/

#include <Xm/XmAll.h>

static GC  gc;
static int x0, y0;  /* Starting point of the line */

/* (A1) When button1 is pressed, initialize the start location of the line */
static void PressAC(w, event, params, num_params)
    Widget    w;           /* Widget ID on which this action was registered */
    XEvent   *event;       /* Event struct which triggers this action */
    String   *params;      /* List of passed parameters (array of strings) */
    Cardinal *num_params;  /* The number of passed parameters (pointer) */
{
    if (event->type == ButtonPress &&        /* The button is pressed  */
        event->xbutton.button == Button1) {  /* pressed button is 1 */
        x0 = event->xbutton.x;  /* Init the X coord. of start of the line */
        y0 = event->xbutton.y;  /* Init the Y coord. of start of the line */
    }
}

/* (A1) If button1 is pressed and moved, trace the movement */
static void MoveAC(w, event, params, num_params)
    Widget    w;
    XEvent   *event;
    String   *params;
    Cardinal *num_params;
{
    int x, y;  /* End point of the line */

    if (event->type == MotionNotify &&        /* Pointer is moved  */
        event->xmotion.state & Button1Mask) { /* Button1 is pressed */
        x = event->xmotion.x;  /* X coord. of end point of this line */
        y = event->xmotion.y;  /* Y coord. of end point of this line */
        /* Draw a line from previous pointer location to current one */
        XDrawLine(XtDisplay(w), XtWindow(w), gc, x0, y0, x, y);
        x0 = x; y0 = y;  /* Set the start point of next line */
    }
}

/* (A1) Clear the window when button3 is clicked */
static void ClearAC(w, event, params, num_params)
    Widget    w;
    XEvent   *event;
    String   *params;
    Cardinal *num_params;
{
    if (event->type == ButtonRelease &&      /* The button is pressed  */
        event->xbutton.button == Button3)    /* pressed button is 3 */
        XClearWindow(XtDisplay(w), XtWindow(w));  /* Clear the window */
}

/* (A2) Define the action table */
static XtActionsRec actions[] = {
    /* { "Action Name",  Action Function} */
       { "press-action", PressAC        },
       { "move-action",  MoveAC         },
       { "clear-action", ClearAC        }
};

/* (T1) (A4) Define the translation as string */
static String trans_list = "\
    <Btn1Down>   : press-action() \n\
    <Btn1Motion> : move-action()  \n\
    <Btn3Up>     : clear-action() ";

main(argc, argv)
    int  argc;
    char **argv;
{
    XtAppContext app_context;
    Widget  toplevel, bboard, canvas;
    Arg     al[20];
    int     ac;
    XtTranslations trans_table;

    XtSetLanguageProc(NULL, NULL, NULL);
    ac = 0;
    toplevel = XtAppInitialize(&app_context, "Actiondraw", NULL, 0,
                               &argc, argv, NULL, al, ac);
    /* (A3) Register the action table */
    XtAppAddActions(app_context, actions, XtNumber(actions));
    /* (T2) Compile the translation string into internal format */
    trans_table = XtParseTranslationTable(trans_list);

    ac = 0;
    bboard = XmCreateBulletinBoard(toplevel, "bboard", al, ac);
    XtManageChild(bboard);
    ac = 0;
    XtSetArg(al[ac], XmNwidth, 300); ac++;
    XtSetArg(al[ac], XmNheight, 300); ac++;
    canvas = XmCreateDrawingArea(bboard, "canvas", al, ac);
    XtManageChild(canvas);

    /* (T3) Delete all translations of drawing area widget
            and replace with new one. */
    XtUninstallTranslations(canvas);
    XtOverrideTranslations(canvas, trans_table);

    XtRealizeWidget(toplevel);
    gc = XCreateGC(XtDisplay(canvas), XtWindow(canvas), NULL, NULL);
    XSetForeground(XtDisplay(canvas), gc, BlackPixelOfScreen(XtScreen(canvas)));
    XtAppMainLoop(app_context);
}
